/* eslint-disable no-console */
const {existsSync, readFileSync} = require('fs')
const {extname} = require('path')
const {getSpawnPromise} = require('@s-ui/helpers/cli')
const {promisify} = require('util')
const exec = promisify(require('child_process').exec)

const GIT_IGNORE_PATH = `${process.cwd()}/.gitignore`
const OPTIONS = {
  staged: '--staged',
  addFixes: '--add-fixes',
  pattern: '--pattern'
}

/**
 * Get lines of a text file as an array of strings
 * @param {String} path
 * @returns {Array<String>}
 */
const getFileLinesAsArray = path =>
  existsSync(path)
    ? readFileSync(path, 'utf8')
        .split('\n')
        .filter(Boolean)
    : []

/**
 * Get as array .gitignore files and filter lines that are comments
 * @returns {Array<String>}
 */
const getGitIgnoredFiles = () =>
  getFileLinesAsArray(GIT_IGNORE_PATH).filter(line => !line.startsWith('#'))

/**
 * Get from git status name of staged files
 * @param {Array<String>} extensions Extensions list: ['js', 'sass', 'css']
 * @returns {Promise<Array>} Array of file paths
 */
const getGitStatusFiles = async extensions => {
  return exec('git', [
    'diff',
    '--cached',
    '--name-only',
    '--diff-filter=d' // --diff-filter=d -> Delete files as excluded
  ]).then(({stdout: summary = ''}) => {
    return summary
      .split('\n')
      .filter(file => extensions.includes(extname(file).substring(1)))
  })
}

/**
 * Get files to lint according to command options
 * @param {Array<String>} extensions Extensions list: ['js', 'sass', 'css']
 * @param {String} defaultFiles Defaults to './'
 * @returns {Promise<Array>} Array of file patterns
 */
const getFilesToLint = async (extensions, defaultFiles = './') =>
  process.argv.includes(OPTIONS.staged)
    ? getGitStatusFiles(extensions)
    : [defaultFiles]

/**
 * If --staged option is on, it will staged made changes
 * @param {Array<String>} extensions Extensions list: ['js', 'sass', 'css']
 * @returns {Promise}
 */
const stageFilesIfRequired = async extensions => {
  const {argv} = process
  if (argv.includes(OPTIONS.staged) && argv.includes(OPTIONS.addFixes)) {
    const files = await getGitStatusFiles(extensions)
    return getSpawnPromise('git', ['add', ...files])
  }
}

/**
 * Get if current process has option set
 * @param {String} option
 */
const isOptionSet = option => process.argv.includes(`--${option}`)

exports.getFileLinesAsArray = getFileLinesAsArray
exports.getFilesToLint = getFilesToLint
exports.getGitIgnoredFiles = getGitIgnoredFiles
exports.stageFilesIfRequired = stageFilesIfRequired
exports.isOptionSet = isOptionSet
exports.GIT_IGNORE_PATH = GIT_IGNORE_PATH
exports.OPTIONS = OPTIONS
